@@ -2,7 +2,7 @@
  *
  *   syscalls.c: syscall mgmt
  *
- *   Copyright 2006-2021 Renzo Davoli University of Bologna - Italy
+ *   Copyright 2006-2023 Renzo Davoli University of Bologna - Italy
  *   Copyright 2005 Andrea Gasparini University of Bologna - Italy
  *
  * This program is free software; you can redistribute it and/or
@@ -45,6 +45,7 @@
 #include <sys/timeb.h>
 #include <sys/mman.h>
 #include <sys/epoll.h>
+#include <sys/sysmacros.h>
 #include <bits/wordsize.h>
 #include <utime.h>
 #include <stdarg.h>
@@ -65,6 +66,7 @@
 sfun _pure_syscall = syscall;
 sfun _pure_native_syscall;
 
+#if 1
 #if defined(__NR_mmap2)
 static int _pageshift()
 {
@@ -94,7 +96,7 @@ long _pure_debug_printf(const char *format, ...)
 }
 
 // open must consider two mode of calling: with two or three arguments
-int __open_2(const char* pathname,int flags){
+static int __purelibc_open_2(const char* pathname,int flags){
 #if defined(__NR_openat) && ! defined(__NR_open)
 	return _pure_syscall(__NR_openat,AT_FDCWD,pathname,flags);
 #else
@@ -102,7 +104,13 @@ int __open_2(const char* pathname,int flags){
 #endif
 }
 
-static int __open_3(const char* pathname,int flags, mode_t mode) {
+#ifndef __USE_FILE_OFFSET64
+int __open_2(const char* pathname,int flags){
+	return __purelibc_open_2(pathname, flags);
+}
+#endif
+
+static int __purelibc_open_3(const char* pathname,int flags, mode_t mode) {
 #if defined(__NR_openat) && ! defined(__NR_open)
 	return _pure_syscall(__NR_openat,AT_FDCWD,pathname,flags,mode);
 #else
@@ -110,6 +118,7 @@ static int __open_3(const char* pathname,int flags, mode_t mode) {
 #endif
 }
 
+#ifndef __USE_FILE_OFFSET64
 int open(const char* pathname,int flags,...){
 	va_list arg_list;
 	if( flags &  O_CREAT ){
@@ -117,11 +126,12 @@ int open(const char* pathname,int flags,...){
 		va_start(arg_list,flags);
 		mode = va_arg(arg_list,mode_t);
 		va_end(arg_list);
-		return __open_3(pathname,flags,mode);
+		return __purelibc_open_3(pathname,flags,mode);
 	}
 	else
-		return __open_2(pathname,flags);
+		return __purelibc_open_2(pathname,flags);
 }
+#endif
 
 int open64(const char* pathname,int flags,...){
 	va_list arg_list;
@@ -130,24 +140,26 @@ int open64(const char* pathname,int flags,...){
 		va_start(arg_list,flags);
 		mode = va_arg(arg_list,mode_t);
 		va_end(arg_list);
-		return __open_3(pathname,flags|O_LARGEFILE,mode);
+		return __purelibc_open_3(pathname,flags|O_LARGEFILE,mode);
 	}
 	else
-		return __open_2(pathname,flags|O_LARGEFILE);
+		return __purelibc_open_2(pathname,flags|O_LARGEFILE);
 }
 
 int __open64_2 (const char* pathname,int flags){
-	return __open_2(pathname,flags|O_LARGEFILE);
+	return __purelibc_open_2(pathname,flags|O_LARGEFILE);
 }
 
+#ifndef __USE_FILE_OFFSET64
 int creat(const char *pathname, mode_t mode)
 {
-	return __open_3(pathname,O_CREAT|O_WRONLY|O_TRUNC,mode);
+	return __purelibc_open_3(pathname,O_CREAT|O_WRONLY|O_TRUNC,mode);
 }
+#endif
 
 int creat64(const char *pathname, mode_t mode)
 {
-	return __open_3(pathname,O_CREAT|O_WRONLY|O_TRUNC|O_LARGEFILE,mode);
+	return __purelibc_open_3(pathname,O_CREAT|O_WRONLY|O_TRUNC|O_LARGEFILE,mode);
 }
 
 int close(int fd){
@@ -211,15 +223,52 @@ int dup3(int oldfd, int newfd, int flags){
 #define __NR_FSTATAT64 __NR_newfstatat
 #endif
 
+#if defined(__NR_statx) && ! defined(__NR_FSTATAT64)
+	#define __NR_FSTATAT64 __NR_statx
+int __purelibc_newstat64(int dirfd, const char *restrict pathname,
+		struct stat *restrict statbuf, int flags) {
+  struct statx buf[1];
+  int rv = _pure_syscall(__NR_statx, dirfd, pathname, flags, STATX_BASIC_STATS, buf);
+  if (rv == 0) {
+    statbuf->st_dev = makedev(buf->stx_dev_major, buf->stx_dev_minor);
+    statbuf->st_ino = buf->stx_ino;
+    statbuf->st_mode = buf->stx_mode;
+    statbuf->st_nlink = buf->stx_nlink;
+    statbuf->st_uid = buf->stx_uid;
+    statbuf->st_gid = buf->stx_gid;
+    statbuf->st_rdev = makedev(buf->stx_rdev_major, buf->stx_rdev_minor);
+    statbuf->st_size = buf->stx_size;
+    statbuf->st_blksize = buf->stx_blksize;
+    statbuf->st_blocks = buf->stx_blocks;
+    statbuf->st_atim.tv_sec = buf->stx_atime.tv_sec;
+    statbuf->st_atim.tv_nsec = buf->stx_atime.tv_nsec;
+    statbuf->st_mtim.tv_sec = buf->stx_mtime.tv_sec;
+    statbuf->st_mtim.tv_nsec = buf->stx_mtime.tv_nsec;
+    statbuf->st_ctim.tv_sec = buf->stx_ctime.tv_sec;
+    statbuf->st_ctim.tv_nsec = buf->stx_ctime.tv_nsec;
+  }
+  return rv;
+}
+#else
+	#define __purelibc_newstat64(dirfd, pathname, statbuf, flags) \
+		_pure_syscall(__NR_FSTATAT64, (dirfd), (pathname), (statbuf), (flags))
+#endif
+
 #if __WORDSIZE == 64 || defined(__ILP32__)
 # if defined(__NR_FSTATAT64) && ! defined(__NR_stat)
-#  define __USE_FSTATAT64
+#  define __USE_NEWSTAT64_STAT
+# endif
+# if defined(__NR_FSTATAT64) && ! defined(__NR_fstat)
+#  define __USE_NEWSTAT64_FSTAT
 # endif
 #	define arch_stat64 stat
 #	define IFNOT64(x)
 #else
 # if defined(__NR_FSTATAT64) && ! defined(__NR_stat64)
-#  define __USE_FSTATAT64
+#  define __USE_NEWSTAT64_STAT
+# endif
+# if defined(__NR_FSTATAT64) && ! defined(__NR_fstat64)
+#  define __USE_NEWSTAT64_FSTAT
 # endif
 #	define arch_stat64 stat64
 #	define IFNOT64(x) x
@@ -228,6 +277,7 @@ int dup3(int oldfd, int newfd, int flags){
 #define INTERNAL_MAKE_NAME(a, b) a ## b
 #define MAKE_NAME(a, b) INTERNAL_MAKE_NAME(a, b)
 
+#ifndef __USE_TIME_BITS64
 static void arch_stat64_2_stat(struct arch_stat64 *from, struct stat *to)
 {
 	if ((void*)from == (void*)to)
@@ -249,15 +299,17 @@ static void arch_stat64_2_stat(struct arch_stat64 *from, struct stat *to)
 
 	return;
 }
+#endif
 
 #if __GNUC_PREREQ (2,33)
+#ifndef __USE_TIME_BITS64
 int stat(const char* pathname, struct stat* buf_stat)
 {
-	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64));)
+	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64)));
 		int rv;
 
-#ifdef __USE_FSTATAT64
-	rv = _pure_syscall(__NR_FSTATAT64, AT_FDCWD, pathname, MAKE_NAME(buf_, arch_stat64), 0);
+#ifdef __USE_NEWSTAT64_STAT
+	rv = __purelibc_newstat64(AT_FDCWD, pathname, MAKE_NAME(buf_, arch_stat64), 0);
 #else
 	rv = _pure_syscall(MAKE_NAME(__NR_, arch_stat64), pathname, MAKE_NAME(buf_, arch_stat64));
 #endif
@@ -269,11 +321,11 @@ int stat(const char* pathname, struct stat* buf_stat)
 
 int lstat(const char* pathname, struct stat* buf_stat)
 {
-	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64));)
+	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64)));
 		int rv;
 
-#ifdef __USE_FSTATAT64
-	rv = _pure_syscall(__NR_FSTATAT64, AT_FDCWD, pathname, MAKE_NAME(buf_, arch_stat64), AT_SYMLINK_NOFOLLOW);
+#ifdef __USE_NEWSTAT64_STAT
+	rv = __purelibc_newstat64(AT_FDCWD, pathname, MAKE_NAME(buf_, arch_stat64), AT_SYMLINK_NOFOLLOW);
 #else
 	rv = _pure_syscall(MAKE_NAME(__NR_l, arch_stat64), pathname, MAKE_NAME(buf_, arch_stat64));
 #endif
@@ -285,33 +337,42 @@ int lstat(const char* pathname, struct stat* buf_stat)
 
 int fstat(int fildes, struct stat* buf_stat)
 {
-	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64));)
+	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64)));
 		int rv;
+#ifdef __USE_NEWSTAT64_FSTAT
+	rv = __purelibc_newstat64(fildes, "", MAKE_NAME(buf_, arch_stat64), AT_EMPTY_PATH);
+#else
 	rv = _pure_syscall(MAKE_NAME(__NR_f, arch_stat64), fildes, MAKE_NAME(buf_, arch_stat64));
+#endif
 	if (rv >= 0)
 		arch_stat64_2_stat(MAKE_NAME(buf_, arch_stat64), buf_stat);
 
 	return rv;
 }
+#endif
 
 int stat64(const char* pathname,struct stat64* buf){
-#ifdef __USE_FSTATAT64
-	return _pure_syscall(__NR_FSTATAT64, AT_FDCWD, pathname, buf, 0);
+#ifdef __USE_NEWSTAT64_STAT
+	return __purelibc_newstat64(AT_FDCWD, pathname, buf, 0);
 #else
 	return _pure_syscall(MAKE_NAME(__NR_, arch_stat64), pathname, buf);
 #endif
 }
 
 int lstat64(const char* pathname,struct stat64* buf){
-#ifdef __USE_FSTATAT64
-  return _pure_syscall(__NR_FSTATAT64, AT_FDCWD, pathname, buf, AT_SYMLINK_NOFOLLOW);
+#ifdef __USE_NEWSTAT64_STAT
+  return __purelibc_newstat64(AT_FDCWD, pathname, buf, AT_SYMLINK_NOFOLLOW);
 #else
   return _pure_syscall(MAKE_NAME(__NR_l, arch_stat64), pathname, buf);
 #endif
 }
 
 int fstat64 (int fildes, struct stat64 *buf){
+#ifdef __USE_NEWSTAT64_FSTAT
+  return __purelibc_newstat64(fildes, "", buf, AT_EMPTY_PATH);
+#else
   return _pure_syscall(MAKE_NAME(__NR_f, arch_stat64), fildes, buf);
+#endif
 }
 
 int mknod(const char *pathname, mode_t mode, dev_t dev) {
@@ -325,14 +386,14 @@ int mknod(const char *pathname, mode_t mode, dev_t dev) {
 #else
 int __xstat(int ver, const char* pathname, struct stat* buf_stat)
 {
-	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64));)
+	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64)));
 	int rv;
 	
 	switch(ver)
 	{
 		case _STAT_VER_LINUX:
-#ifdef __USE_FSTATAT64
-			rv = _pure_syscall(__NR_FSTATAT64, AT_FDCWD, pathname, MAKE_NAME(buf_, arch_stat64), 0);
+#ifdef __USE_NEWSTAT64_STAT
+			rv = __purelibc_newstat64(AT_FDCWD, pathname, MAKE_NAME(buf_, arch_stat64), 0);
 #else
 			rv = _pure_syscall(MAKE_NAME(__NR_, arch_stat64), pathname, MAKE_NAME(buf_, arch_stat64));
 #endif
@@ -351,14 +412,14 @@ int __xstat(int ver, const char* pathname, struct stat* buf_stat)
 
 int __lxstat(int ver, const char* pathname, struct stat* buf_stat)
 {
-	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64));)
+	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64)));
 	int rv;
 	
 	switch(ver)
 	{
 		case _STAT_VER_LINUX:
-#ifdef __USE_FSTATAT64
-			rv = _pure_syscall(__NR_FSTATAT64, AT_FDCWD, pathname, MAKE_NAME(buf_, arch_stat64), AT_SYMLINK_NOFOLLOW);
+#ifdef __USE_NEWSTAT64_STAT
+			rv = __purelibc_newstat64(AT_FDCWD, pathname, MAKE_NAME(buf_, arch_stat64), AT_SYMLINK_NOFOLLOW);
 #else
 			rv = _pure_syscall(MAKE_NAME(__NR_l, arch_stat64), pathname, MAKE_NAME(buf_, arch_stat64));
 #endif
@@ -377,12 +438,16 @@ int __lxstat(int ver, const char* pathname, struct stat* buf_stat)
 
 int __fxstat(int ver, int fildes, struct stat* buf_stat)
 {
-	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64));)
+	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64)));
 	int rv;
 	switch(ver)
 	{
 		case _STAT_VER_LINUX:
+#ifdef __USE_NEWSTAT64_FSTAT
+			rv = __purelibc_newstat64(fildes, "", MAKE_NAME(buf_, arch_stat64), AT_EMPTY_PATH);
+#else
 			rv = _pure_syscall(MAKE_NAME(__NR_f, arch_stat64), fildes, MAKE_NAME(buf_, arch_stat64));
+#endif
 			break;
 
 		default:
@@ -396,37 +461,41 @@ int __fxstat(int ver, int fildes, struct stat* buf_stat)
 }
 
 int __xstat64(int ver,const char* pathname,struct stat64* buf){
-#ifdef __USE_FSTATAT64
-	return _pure_syscall(__NR_FSTATAT64, AT_FDCWD, pathname, buf, 0);
+#ifdef __USE_NEWSTAT64_STAT
+	return __purelibc_newstat64(AT_FDCWD, pathname, buf, 0);
 #else
 	return _pure_syscall(MAKE_NAME(__NR_, arch_stat64), pathname, buf);
 #endif
 }
 
 int __lxstat64(int ver,const char* pathname,struct stat64* buf){
-#ifdef __USE_FSTATAT64
-	return _pure_syscall(__NR_FSTATAT64, AT_FDCWD, pathname, buf,	AT_SYMLINK_NOFOLLOW);
+#ifdef __USE_NEWSTAT64_STAT
+	return __purelibc_newstat64(AT_FDCWD, pathname, buf,	AT_SYMLINK_NOFOLLOW);
 #else
 	return _pure_syscall(MAKE_NAME(__NR_l, arch_stat64), pathname, buf);
 #endif
 }
 
 int __fxstat64 (int ver, int fildes, struct stat64 *buf){
+#ifdef __USE_NEWSTAT64_FSTAT
+	return __purelibc_newstat64(fildes, "", buf,	AT_EMPTY_PATH);
+#else
 	return _pure_syscall(MAKE_NAME(__NR_f, arch_stat64), fildes, buf);
+#endif
 }
 
 #ifdef __NR_FSTATAT64
 int __fxstatat64 (int ver, int dirfd, const char *pathname, struct stat64 *buf, int flags){
-	return _pure_syscall(__NR_FSTATAT64,dirfd,pathname,buf,flags);
+	return __purelibc_newstat64(dirfd,pathname,buf,flags);
 }
 int __fxstatat(int ver, int fildes, const char *pathname, struct stat* buf_stat,int flags)
 {
-	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64));)
+	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64)));
 	int rv;
 	switch(ver)
 	{
 		case _STAT_VER_LINUX:
-			rv = _pure_syscall(__NR_FSTATAT64, fildes, pathname, MAKE_NAME(buf_, arch_stat64), flags);
+			rv = __purelibc_newstat64(fildes, pathname, MAKE_NAME(buf_, arch_stat64), flags);
 			break;
 
 		default:
@@ -452,7 +521,7 @@ int __xmknod (int ver, const char *path, mode_t mode, dev_t *dev) {
 
 #ifdef __NR_FSTATAT64
 int fstatat(int dirfd, const char *pathname, struct stat *buf, int flags) {
-	return _pure_syscall(__NR_FSTATAT64,dirfd,pathname,buf,flags);
+	return __purelibc_newstat64(dirfd,pathname,buf,flags);
 }
 #endif
 
@@ -501,7 +570,7 @@ int rmdir(const char* pathname){
 
 int chmod(const char* pathname,mode_t mode){
 #if defined(__NR_fchownat) && ! defined(__NR_chmod)
-	return _pure_syscall(__NR_fchownat,AT_FDCWD,pathname,mode,0);
+	return _pure_syscall(__NR_fchmodat,AT_FDCWD,pathname,mode,0);
 #else
 	return _pure_syscall(__NR_chmod,pathname,mode);
 #endif
@@ -595,16 +664,20 @@ int utime(const char* pathname,const struct utimbuf *buf){
 }
 
 #ifdef __NR_pread
+#ifndef __USE_FILE_OFFSET64
 ssize_t pread(int fs,void* buf, size_t count, __off_t offset){
 	return _pure_syscall(__NR_pread,fs,buf,count,offset);
 }
 #endif
+#endif
 
 #ifdef __NR_pwrite
+#ifndef __USE_FILE_OFFSET64
 ssize_t pwrite(int fs,const void* buf, size_t count, __off_t offset){
 	return _pure_syscall(__NR_pwrite,fs,buf,count,offset);
 }
 #endif
+#endif
 
 #ifdef __NR_pread64
 ssize_t pread64(int fs,void* buf, size_t count, __off64_t offset){
@@ -612,12 +685,14 @@ ssize_t pread64(int fs,void* buf, size_t count, __off64_t offset){
 #if defined(__powerpc__) || defined(__arm__)
 			0,
 #endif
-			__LONG_LONG_PAIR( (__off_t)(offset>>32),(__off_t)(offset&0xffffffff)));
+			__LONG_LONG_PAIR((__off_t)(offset>>32),(__off_t)(offset&0xffffffff)));
 }
+#ifndef __USE_FILE_OFFSET64
 ssize_t pread(int fs,void* buf, size_t count, __off_t offset){
 	return pread64(fs,buf,count,(__off64_t)offset);
 }
 #endif
+#endif
 
 #ifdef __NR_pwrite64
 ssize_t pwrite64(int fs,const void* buf, size_t count, __off64_t offset){
@@ -625,12 +700,14 @@ ssize_t pwrite64(int fs,const void* buf, size_t count, __off64_t offset){
 #if defined(__powerpc__) || defined(__arm__)
 			0,
 #endif
-			__LONG_LONG_PAIR( (__off_t)(offset>>32),(__off_t)(offset&0xffffffff)));
+			__LONG_LONG_PAIR((__off_t)(offset>>32),(__off_t)(offset&0xffffffff)));
 }
+#ifndef __USE_FILE_OFFSET64
 ssize_t pwrite(int fs,const void* buf, size_t count, __off_t offset){
 	return pwrite64(fs,buf,count,(__off64_t)offset);
 }
 #endif
+#endif
 
 #ifdef __NR_preadv
 ssize_t preadv64(int fs,const struct iovec *iov, int iovcnt, __off64_t offset){
@@ -639,7 +716,7 @@ ssize_t preadv64(int fs,const struct iovec *iov, int iovcnt, __off64_t offset){
 #if defined(__powerpc__) || defined(__arm__)
 			0,
 #endif
-			__LONG_LONG_PAIR( (__off_t)(offset>>32),(__off_t)(offset&0xffffffff))
+			__LONG_LONG_PAIR((__off_t)(offset>>32),(__off_t)(offset&0xffffffff))
 #else
 			offset
 #endif
@@ -666,10 +743,12 @@ ssize_t preadv64(int fs,const struct iovec *iov, int iovcnt, __off64_t offset){
 	return rv;
 }
 
+#ifndef __USE_FILE_OFFSET64
 ssize_t preadv(int fs,const struct iovec *iov, int iovcnt, __off_t offset){
 	return preadv64(fs,iov,iovcnt,(__off64_t)offset);
 }
 #endif
+#endif
 
 #ifdef __NR_pwritev
 ssize_t pwritev64(int fs,const struct iovec *iov, int iovcnt, __off64_t offset){
@@ -678,7 +757,7 @@ ssize_t pwritev64(int fs,const struct iovec *iov, int iovcnt, __off64_t offset){
 #if defined(__powerpc__) || defined(__arm__)
 			0,
 #endif
-			__LONG_LONG_PAIR( (__off_t)(offset>>32),(__off_t)(offset&0xffffffff))
+			__LONG_LONG_PAIR((__off_t)(offset>>32),(__off_t)(offset&0xffffffff))
 #else
 			offset
 #endif
@@ -702,10 +781,12 @@ ssize_t pwritev64(int fs,const struct iovec *iov, int iovcnt, __off64_t offset){
 	return rv;
 }
 
+#ifndef __USE_FILE_OFFSET64
 ssize_t pwritev(int fs,const struct iovec *iov, int iovcnt, __off_t offset){
 	return pwritev64(fs,iov,iovcnt,(__off64_t)offset);
 }
 #endif
+#endif
 
 /* getdents has no libc wrapper.
 	 libc uses getdents64 only on 64 bit archs */
@@ -722,9 +803,11 @@ __ssize_t getdents64(int fd, void *dirp, size_t count){
 	return _pure_syscall(__NR_getdents64, fd, dirp, count);
 }
 
+#ifndef __USE_FILE_OFFSET64
 __off_t lseek(int fd,__off_t offset,int whence){
 	return _pure_syscall(__NR_lseek,fd,offset,whence);
 }
+#endif
 
 #ifndef __NR__llseek
 off64_t lseek64(int fd, off64_t offset, int whence) {
@@ -832,6 +915,7 @@ int ioctl(int fd,unsigned long int request, ...){
 	return rv;
 }
 
+#ifndef __USE_TIME_BITS64
 int fcntl(int fd, int cmd, ...){
 	va_list ap;
 	long int arg1;
@@ -854,6 +938,7 @@ int fcntl(int fd, int cmd, ...){
 	return _pure_syscall(__NR_fcntl,fd,cmd,arg1,arg2);
 #endif
 }
+#endif
 
 #ifdef __NR_fcntl64
 int fcntl64(int fd, int cmd, ...){
@@ -1016,7 +1101,7 @@ pid_t fork(void){
 		return -1;
 	else
 		return child_tid;
-#elif defined(__aarch64__) || defined(__riscv) && __riscv_xlen==64
+#elif defined(__aarch64__) || (defined(__riscv) && __riscv_xlen==64) || defined(__loongarch__)
 	int child_tid;
 	if (_pure_syscall(__NR_clone, NULL, CLONE_CHILD_CLEARTID|CLONE_CHILD_SETTID|SIGCHLD, &child_tid) < 0)
 		return -1;
@@ -1066,7 +1151,8 @@ int nice(int inc){
 	defined(__alpha__) || defined(__s390x__) || \
 	(defined(__mips__) && defined(__LP64__)) || \
 	defined(__aarch64__) || \
-	(defined(__riscv) && __riscv_xlen==64)
+	(defined(__riscv) && __riscv_xlen==64) || \
+	defined(__loongarch__)
 	int nice = _pure_syscall(__NR_getpriority,PRIO_PROCESS,0);
 	return _pure_syscall(__NR_setpriority,PRIO_PROCESS,0,nice + inc);
 #else
@@ -1127,8 +1213,8 @@ int getrlimit(__rlimit_resource_t resource, struct rlimit *rlim){
 }
 #endif
 
-int getrusage(int who, struct rusage *usage){
-	return _pure_syscall(__NR_getrusage,usage);
+int getrusage(__rusage_who_t who, struct rusage *usage){
+	return _pure_syscall(__NR_getrusage, who, usage);
 }
 
 #ifdef GETTIMEOFDAY_TZ
@@ -1195,7 +1281,7 @@ int select(int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct t
 #if defined(__x86_64__) || defined(__s390x__) || \
 	defined(__alpha__) || defined(__ia64__)
 	return _pure_syscall(__NR_select,n,readfds,writefds,exceptfds,timeout);
-#elif defined(__aarch64__) || defined(__riscv) && __riscv_xlen==64
+#elif defined(__aarch64__) || (defined(__riscv) && __riscv_xlen==64) || defined(__loongarch__)
 	if (timeout == NULL)
 		return pselect(n,readfds,writefds,exceptfds,NULL,NULL);
 	else {
@@ -1248,13 +1334,17 @@ int epoll_pwait(int epfd, struct epoll_event *events,
 }
 #endif
 
+#ifndef __USE_FILE_OFFSET64
 int truncate(const char *path, __off_t length){
 	return _pure_syscall(__NR_truncate,path,length);
 }
+#endif
 
+#ifndef __USE_FILE_OFFSET64
 int ftruncate(int fd, __off_t length){
 	return _pure_syscall(__NR_ftruncate,fd,length);
 }
+#endif
 
 #ifdef __NR_truncate64
 int truncate64(const char *path, __off64_t length){
@@ -1262,7 +1352,7 @@ int truncate64(const char *path, __off64_t length){
 #if defined(__powerpc__)
 			0,
 #endif
-			__LONG_LONG_PAIR( (__off_t)(length>>32),(__off_t)(length&0xffffffff)));
+			__LONG_LONG_PAIR((__off_t)(length>>32),(__off_t)(length&0xffffffff)));
 }
 #endif
 
@@ -1272,7 +1362,7 @@ int ftruncate64(int fd, __off64_t length){
 #if defined(__powerpc__)
 			0,
 #endif
-			__LONG_LONG_PAIR( (__off_t)(length>>32),(__off_t)(length&0xffffffff)));
+			__LONG_LONG_PAIR((__off_t)(length>>32),(__off_t)(length&0xffffffff)));
 }
 #endif
 
@@ -1284,13 +1374,17 @@ int setpriority(__priority_which_t which, id_t who, int prio){
 	return _pure_syscall(__NR_setpriority,which,who,prio);
 }
 
+#ifndef __USE_FILE_OFFSET64
 int statfs(const char *path, struct statfs *buf){
 	return _pure_syscall(__NR_statfs,path,buf);
 }
+#endif
 
+#ifndef __USE_FILE_OFFSET64
 int fstatfs(int fd, struct statfs *buf){
 	return _pure_syscall(__NR_fstatfs,fd,buf);
 }
+#endif
 
 #ifdef __NR_statfs64
 /* LIBC add an extra arg: the buf size */
@@ -1584,6 +1678,7 @@ int ftime(struct timeb *tp){
 /* *at syscalls */
 
 #ifdef __NR_openat
+#ifndef __USE_FILE_OFFSET64
 int openat(int dirfd,const char* pathname,int flags,...){
 	va_list arg_list;
 	if( flags &  O_CREAT ){
@@ -1596,6 +1691,7 @@ int openat(int dirfd,const char* pathname,int flags,...){
 	else
 		return _pure_syscall(__NR_openat,dirfd,pathname,flags);
 }
+#endif
 
 int openat64(int dirfd,const char* pathname,int flags,...){
 	va_list arg_list;
@@ -1610,10 +1706,12 @@ int openat64(int dirfd,const char* pathname,int flags,...){
 		return _pure_syscall(__NR_openat,dirfd,pathname,flags|O_LARGEFILE);
 }
 
+#ifndef __USE_FILE_OFFSET64
 int	__openat_2(int dirfd, const char *pathname, int flags)
 {
 	return _pure_syscall(__NR_openat,dirfd,pathname,flags);
 }
+#endif
 
 int	__openat64_2(int dirfd, const char *pathname, int flags)
 {
@@ -1690,7 +1788,13 @@ int fchmodat(int dirfd, const char *pathname, mode_t mode, int flags){
 #ifdef __NR_faccessat
 //The raw faccessat() system call takes only the first  three  arguments
 // this function has been inspired by glibc: sysdeps/unix/sysv/linux/faccessat.c
+// if the kernel supports faccessat2 use it!
 int faccessat(int dirfd, const char *pathname, int mode, int flags){
+#ifdef __NR_faccessat2
+	int ret = _pure_syscall(__NR_faccessat2, dirfd, pathname, mode, flags);
+	if (ret == 0 || errno != ENOSYS)
+    return ret;
+#endif
 	if (flags & ~(AT_EACCESS | AT_SYMLINK_NOFOLLOW)) {
     errno = EINVAL;
     return -1;
@@ -1813,6 +1917,7 @@ sfun _pure_start(sfun pure_syscall, int flags)
 	return _pure_native_syscall;
 }
 
+#endif
 /* this is convenient since casting the return value of dlsym() to
  * a function pointer erroneously procudes a warning */
 #pragma GCC diagnostic ignored "-Wpedantic"
